import 'package:pigeon/pigeon.dart';

@HostApi()
abstract class FlutterAuthApi {
  // Auth core
  CommonAuthResult getInstance(String appName);

  @async
  IsAuthorizedResult isAuthorized(String appName);

  @async
  AccessTokenResult getCurrentToken(String appName);

  @async
  AccessTokenResult refreshToken(String appName);

  @async
  IdTokenResult exchangeToken(String appName, String iamAudience);

  // Auth login
  @async
  CommonAuthResult requestOtp(String appName, String phoneNumber);

  @async
  CommonAuthResult loginWithGoogleCredential(String appName, GoogleCredential credential);

  @async
  CommonAuthResult loginWithFacebookCredential(String appName, FacebookCredential credential);

  @async
  CommonAuthResult loginWithAppleCredential(String appName, AppleCredential credential);

  @async
  CommonAuthResult loginWithUsernamePasswordCredential(
      String appName, UsernamePasswordCredential credential);

  @async
  CommonAuthResult loginWithPhoneOtpCredential(String appName, PhoneOtpCredential credential);

  @async
  CommonAuthResult loginWithCustomCredential(String appName, CustomCredential credential);

  @async
  CommonAuthResult loginWithGoogle(String appName);

  @async
  CommonAuthResult loginWithFacebook(String appName);

  @async
  CommonAuthResult loginWithApple(String appName);

  @async
  LogoutResult logout(String appName);

  @async
  UserInfoResult getUserInfo(String appName);

  @async
  CommonAuthResult verifyPhone(String appName, String phone);

  @async
  CommonAuthResult verifyEmail(String appName, String email);

  @async
  CommonAuthResult confirmPhone(String appName, String phone, String code);

  @async
  CommonAuthResult confirmEmail(String appName, String email, String code);
}

// Model
class CommonAuthResult {
  bool? isSuccess;
  String? error;
}

class IsAuthorizedResult {
  bool? isAuthorized;
  bool? isSuccess;
  String? error;
}

class LogoutResult {
  bool? isLoggedOut;
  bool? isSuccess;
  String? error;
}

class AccessTokenResult {
  bool? isSuccess;
  String? error;
  IamToken? iamToken;
}

class IdTokenResult {
  bool? isSuccess;
  String? error;
  IdToken? idToken;
}

class IamToken {
  String? id;
  String? accessToken;
  String? refreshToken;
  int? expiresIn;
  String? tokenType;
}

class IdToken {
  String? idToken;
  String? accessToken;
  String? refreshToken;
  int? expiresIn;
  String? tokenType;
}

class AppleCredential {
  String? accessToken;
}

class GoogleCredential {
  String? accessToken;
}

class FacebookCredential {
  String? accessToken;
}

class PhoneOtpCredential {
  String? phoneNumber;
  String? otp;
}

class UsernamePasswordCredential {
  String? username;
  String? password;
}

class CustomCredential {
  String? idToken;
}

// User
class UserInfoResult {
  UserInfo? userInfo;
  bool? isSuccess;
  String? error;
}

class UserInfo {
  String? sub;
  String? name;
  String? picture;
  String? email;
  String? phoneNumber;
  String? birthday;
  String? address;
  List<String?>? roles;
  List<String?>? permissions;
  Map<String?, Object?>? metadata;
}
